#include "ks0108.h"

unsigned char LCD_Read_Data(void);
unsigned int abs(int);

volatile unsigned char lcd_x = 0, lcd_y = 0;


void LCD_Init(void)
{
   LCD_CONTROL_DDR = 0xFF;
   LCD_DATA_DDR = 0xFF;
   LCD_CONTROL_PORT &= ~(1<<LCD_RESET);
   _delay_loop_2(20);
   LCD_CONTROL_PORT |= (1<<LCD_RESET);
   
   SET_CS1();
   CLEAR_CS2();
   LCD_Send_Cmd(DISPLAY_ON);
   
   SET_CS2();
   CLEAR_CS1();
   LCD_Send_Cmd(DISPLAY_ON);
   SET_CS1();
   SET_CS2();
}

void LCD_Wait(void)
{
   LCD_DATA_DDR = 0x00;             // Set DDR to input.
   CLEAR_RS();               // Set Instruction/Data pin to low
   SET_RW();               // Set Read/Write to Read (high)
   SET_ENABLE();
   
   while((LCD_DATA_PIN & DISPLAY_STATUS_BUSY));    // Do loop until LCD isn't busy anymore
   
   CLEAR_ENABLE();
}

void LCD_Send_Cmd(unsigned char cmd)
{
   LCD_Wait();          // Wait for LCD to be ready for commands
   CLEAR_RS();          // Set Instruction/Data pin to low
   CLEAR_RW();          // Set Read/Write to Write (low)
   
   LCD_DATA_DDR = 0xFF;      // Set DDR to output
   LCD_DATA_PORT = cmd;      // Send the data
   
   SET_ENABLE();         // Pulse Enable
   _delay_loop_2(20);      
   CLEAR_ENABLE();
}

void LCD_Send_Data(unsigned char data)
{
   LCD_Wait();          // Wait for LCD to be ready for commands
   SET_RS();             // Set Instruction/Data pin to high
   CLEAR_RW();          // Set Read/Write to Write (low)
   
   LCD_DATA_DDR = 0xFF;       // Set DDR to output
   LCD_DATA_PORT = data;      // Send data
   
   SET_ENABLE();         // Pulse Enable
   _delay_loop_2(20);      
   CLEAR_ENABLE();
}

unsigned char LCD_Read_Data(void)
{
   LCD_Wait();             // Wait for LCD to be ready for commands
   LCD_DATA_DDR = 0x00;          // Set DDR to output
   
   SET_RS();             // Set Instruction/Data pin to high
   SET_RW();             // Set Read/Write to Read (high)
   SET_ENABLE();            // Set Enable high
   _delay_loop_2(20);   
   CLEAR_ENABLE();            // Set Enable low
   
   unsigned char data = LCD_DATA_PIN;   // Get data
   
   return data;
}

void LCD_Set_Pos(unsigned char x, unsigned char y)
{
   lcd_x = x;
   lcd_y = y;
   
   unsigned char tmp = 0;
   
   if (lcd_x > 63)
   {
      SET_CS2();
      CLEAR_CS1();
      tmp = (lcd_x - 64);
   }
   
   else
   {
      SET_CS1();
      CLEAR_CS2();
      tmp = lcd_x;
   }
   
   LCD_Send_Cmd(ADDRESS_X | tmp);
   LCD_Send_Cmd(ADDRESS_Y | lcd_y);

   SET_CS2();
   CLEAR_CS1();
   LCD_Send_Cmd(DISPLAY_LINE);
   
   SET_CS1();
   CLEAR_CS2();
   LCD_Send_Cmd(DISPLAY_LINE);
   
   if (lcd_x > 63)
   {
      SET_CS2();
      CLEAR_CS1();
   }
   
   else
   {
      SET_CS1();
      CLEAR_CS2();
   }   
}

void LCD_Draw_Dot(unsigned char x, unsigned char y)
{
   LCD_Set_Pos(x, y/8);
   unsigned char tempdata = LCD_Read_Data();
   tempdata = LCD_Read_Data();
   LCD_Set_Pos(x, y/8);
   LCD_Send_Data(tempdata | (1<<(y % 8)));
}

void LCD_Clear_Dot(unsigned char x, unsigned char y)
{
   LCD_Set_Pos(x, y/8);
   unsigned char tempdata = LCD_Read_Data();
   tempdata = LCD_Read_Data();
   LCD_Set_Pos(x, y/8);
   LCD_Send_Data(tempdata & ~(1<<(y % 8)));

}

void LCD_Fill_Screen(void)
{
   for(unsigned char y = 0; y < 8; y++)
   {
      for(unsigned char x = 0; x < 128; x++)
      {
         LCD_Set_Pos(x, y);
         LCD_Send_Data(0xFF);
      }
   }
}

void LCD_Clear_Screen(void)
{
   for(unsigned char y = 0; y < 8; y++)
   {
      for(unsigned char x = 0; x < 128; x++)
      {
         LCD_Set_Pos(x, y);
         LCD_Send_Data(0x00);
      }
   }
}

void LCD_Draw_Line(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char clr)
{
   char deltax = abs(x2 - x1);
   char deltay = abs(y2 - y1);
   char x = x1;
   char y = y1;
   char xinc1;
   char xinc2;
   char yinc1;
   char yinc2;
   char den;
   char num;
   char numadd;
   char numpixels;

   if (x2 >= x1)
   {
      xinc1 = 1;
      xinc2 = 1;
   }
   
   else
   {
      xinc1 = -1;
      xinc2 = -1;
   }

   if (y2 >= y1)
   {
      yinc1 = 1;
      yinc2 = 1;
   }
   
   else
   {
      yinc1 = -1;
      yinc2 = -1;
   }

   if (deltax >= deltay)
   {
      xinc1 = 0;
      yinc2 = 0;
      den = deltax;
      num = deltax / 2;
      numadd = deltay;
      numpixels = deltax;
   }
   
   else
   {
      xinc2 = 0;
      yinc1 = 0;
      den = deltay;
      num = deltay / 2;
      numadd = deltax;
      numpixels = deltay;
   }

   for (unsigned char curpixel = 0; curpixel <= numpixels; curpixel++)
   {
      if (clr)
      {
         LCD_Clear_Dot(x, y);
      }
      
      else
      {
         LCD_Draw_Dot(x, y);
      }
      
      num += numadd;
      
      if (num >= den)
      {
         num -= den;
         x += xinc1;
         y += yinc1;
      }
      
      x += xinc2;
      y += yinc2;
   }
}

void LCD_Draw_Rect(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char fill, unsigned char clr)
{
   if (fill != 0)
   {
      for (int i = y1; i <= y2; i++)
      {
         LCD_Draw_Line(x1, i, x2, i, clr);
      }
   }
   
   else
   {
      LCD_Draw_Line(x1, y1, x2, y1, clr);
      LCD_Draw_Line(x1, y2, x2, y2, clr);
      LCD_Draw_Line(x2, y1, x2, y2, clr);
      LCD_Draw_Line(x1, y1, x1, y2, clr);
   }
}

void LCD_Draw_Circle(unsigned char x1, unsigned char y1, unsigned char radius)
{
   int f = 1 - radius;
   int ddF_x = 0;
   int ddF_y = -2 * radius;
   int x = 0;
   int y = radius;

   LCD_Draw_Dot(x1, y1 + radius);
   LCD_Draw_Dot(x1, y1 - radius);
   LCD_Draw_Dot(x1 + radius, y1);
   LCD_Draw_Dot(x1 - radius, y1);

   while(x < y)
   {
      if(f >= 0)
      {
         y--;
         ddF_y += 2;
         f += ddF_y;
      }
      
      x++;
      ddF_x += 2;
      f += ddF_x + 1;   
      LCD_Draw_Dot(x1 + x, y1 + y);
      LCD_Draw_Dot(x1 - x, y1 + y);
      LCD_Draw_Dot(x1 + x, y1 - y);
      LCD_Draw_Dot(x1 - x, y1 - y);
      LCD_Draw_Dot(x1 + y, y1 + x);
      LCD_Draw_Dot(x1 - y, y1 + x);
      LCD_Draw_Dot(x1 + y, y1 - x);
      LCD_Draw_Dot(x1 - y, y1 - x);
   }
}

unsigned int abs(int num)
{
   if (num >= 0)
   {
      return num;
   }
   
   else
   {
      return -num;
   }
}

static uint8_t TurnBitsAround( uint8_t aByte ){
return (aByte & 0x80 ? 0x01 : 0) |
       (aByte & 0x40 ? 0x02 : 0) |
       (aByte & 0x20 ? 0x04 : 0) |
       (aByte & 0x10 ? 0x08 : 0) |
       (aByte & 0x08 ? 0x10 : 0) |
       (aByte & 0x04 ? 0x20 : 0) |
       (aByte & 0x02 ? 0x40 : 0) |
       (aByte & 0x01 ? 0x80 : 0);
} 


void LCD_Write_Char(unsigned char x, unsigned char y, char ch)
{   
   
   if (ch == ' '){
      lcd_x += 4;
      return;
   }
   if(ch == '1'){
	  x += 1;
   }
   if(ch == 'I'){
  	  x += 1;
   }
   if(ch == 'i'){
      x += 1;
   }


   unsigned char ch_data[5] = {0, 0, 0, 0, 0};
	ch_data[0] = TurnBitsAround(smallFont[(ch-32)*5+0]);
	ch_data[1] = TurnBitsAround(smallFont[(ch-32)*5+1]);
	ch_data[2] = TurnBitsAround(smallFont[(ch-32)*5+2]);
	ch_data[3] = TurnBitsAround(smallFont[(ch-32)*5+3]);
	ch_data[4] = TurnBitsAround(smallFont[(ch-32)*5+4]);
   //if (ch_data[0] != 0)
   //{
      for(unsigned char a = 0; a < sizeof(ch_data); a++)
      {
         for(unsigned char i = 0; i < 8; i++)
         {
            if ((ch_data[a] == 0) || ((x + a) == 127))
              break;
               
            if (ch_data[a] & 0x80)
               LCD_Draw_Dot((x + a), (y + i));
               
            ch_data[a] <<= 1;
         }
      }
   //}
}

void LCD_Write_String(unsigned char x, unsigned char y, char *s)
{
   unsigned char i = 0;
   
   while(s[i])
   {
      LCD_Write_Char(x, y, s[i]);
      x = (lcd_x + 2);
      
      if (lcd_x == 127)
      {
         return;
      }
      
      i++;
   }
}
